#import <Cocoa/Cocoa.h>
#include <pthread.h>

extern "C" int32_t CMIOGraphInitialize(void *graph);
extern "C" int32_t CMIOGraphUninitialize(void *graph);

class nsAutoreleasePool {
public:
    nsAutoreleasePool()
    {
        mLocalPool = [[NSAutoreleasePool alloc] init];
    }
    ~nsAutoreleasePool()
    {
        [mLocalPool release];
    }
private:
    NSAutoreleasePool *mLocalPool;
};

static int32_t Hooked_CMIOGraphInitialize(void *graph)
{
  int32_t retval = ::CMIOGraphInitialize(graph);
  nsAutoreleasePool localPool;
  NSLog(@"CMIOGraphInitialize(): graph %p, thread %p, returning %i",
        graph, pthread_self(), retval); 
  return retval;
}

static int32_t Hooked_CMIOGraphUninitialize(void *graph)
{
  //int32_t retval = ::CMIOGraphUninitialize(graph);
  int32_t retval = 0;
  nsAutoreleasePool localPool;
  NSLog(@"CMIOGraphUninitialize(): graph %p, thread %p, returning %i",
        graph, pthread_self(), retval); 
  return retval;
}

#pragma mark -

struct interpose_substitution {
  const void* replacement;
  const void* original;
};

#define INTERPOSE_FUNCTION(function) \
    { reinterpret_cast<const void*>(Hooked_##function), \
      reinterpret_cast<const void*>(function) }

__attribute__((used)) static const interpose_substitution substitutions[]
    __attribute__((section("__DATA, __interpose"))) = {
  INTERPOSE_FUNCTION(CMIOGraphInitialize),
  INTERPOSE_FUNCTION(CMIOGraphUninitialize),
};
